<?php namespace Webmors\Catalog;

use System\Classes\PluginBase;
use Webmors\Catalog\Models\Category;
use Webmors\Catalog\Models\Product;

class Plugin extends PluginBase
{
    public function registerComponents()
    {
    }

    public function registerSettings()
    {
    }

    public function boot()
    {
        // Add model to Static Pages menuitem type
        \Event::listen('pages.menuitem.listTypes', function () {
            return [
                'webmors-catalog' => 'Webmors Catalog Category',
            ];
        });
        
        \Event::listen('pages.menuitem.getTypeInfo', function ($type) {
            if ($type === 'webmors-catalog') {
                $theme = \Cms\Classes\Theme::getActiveTheme();
                $pages = \Cms\Classes\Page::listInTheme($theme, true);
                return [
                    'references' => Category::resolveMenuReferences(),
                    'dynamicItems' => true,
                    'cmsPages' => $pages,
                ];
            }
        });
        
        \Event::listen('pages.menuitem.resolveItem', function ($type, $item, $url, $theme) {
            if ($type === 'webmors-catalog') {
                return Product::resolveMenuItem($item, $url, $theme);
            }
        });

   		\Event::listen('offline.sitesearch.query', function ($query) {
   			// The controller is used to generate page URLs.
        	$controller = \Cms\Classes\Controller::getController() ?? new \Cms\Classes\Controller();

        	// Search your plugin's contents
	        $items = Models\Product
	            ::where('title', 'like', "%${query}%")
	            ->orWhere('description', 'like', "%${query}%")
	            ->get();

	      	$items2 = Models\Category
	      		::where('name', 'like', "%${query}%")
	            ->get();

	        // Now build a results array
	        $results = $items->map(function ($item) use ($query, $controller) {

	            // If the query is found in the title, set a relevance of 2
	            $relevance = mb_stripos($item->title, $query) !== false ? 2 : 1;
	            
	            // Optional: Add an age penalty to older results. This makes sure that
	            // newer results are listed first.
	            // if ($relevance > 1 && $item->created_at) {
	            //    $ageInDays = $item->created_at->diffInDays(\Illuminate\Support\Carbon::now());
	            //    $relevance -= \OFFLINE\SiteSearch\Classes\Providers\ResultsProvider::agePenaltyForDays($ageInDays);
	            // }

	            return [
	                'provider'     => 'Property',
	                'title'     => $item->title,
	                'text'      => $item->description,
	                'url'       => $controller->pageUrl('product.htm', ['slug' => $item->slug]),
	                //'thumb'     => optional($item->images)->first(), // Instance of System\Models\File
	                'relevance' => $relevance, // higher relevance results in a higher
	                                           // position in the results listing
	                //'meta' => 'data',       // optional, any other information you want
	                                           // to associate with this result
	                'model' => $item       // optional, pass along the original model
	            ];
	        });

	        // Now build a results array
	        $results2 = $items2->map(function ($item) use ($query, $controller) {

	            // If the query is found in the name, set a relevance of 2
	            $relevance = mb_stripos($item->name, $query) !== false ? 2 : 1;
	            
	            // Optional: Add an age penalty to older results. This makes sure that
	            // newer results are listed first.
	            // if ($relevance > 1 && $item->created_at) {
	            //    $ageInDays = $item->created_at->diffInDays(\Illuminate\Support\Carbon::now());
	            //    $relevance -= \OFFLINE\SiteSearch\Classes\Providers\ResultsProvider::agePenaltyForDays($ageInDays);
	            // }

	            return [
	                'provider'     => 'Category',
                    'title'     => $item->name,
                    'text'      => 'Lorem ipsum dolor site amet',
	                'url'       => $controller->pageUrl('category.htm', ['slug' => $item->slug]),
	                //'thumb'     => optional($item->images)->first(), // Instance of System\Models\File
	                'relevance' => $relevance, // higher relevance results in a higher
	                                           // position in the results listing
	                //'meta' => 'data',       // optional, any other information you want
	                                           // to associate with this result
	                'model' => $item       // optional, pass along the original model
	            ];
	        });

	        $combinedResultsArray = array_merge($results->toArray(), $results2->toArray());

	        return [
	           	'results'  => $combinedResultsArray
	        ];

   		});
    }
}
