<?php namespace Webmors\Catalog\Models;

use Model;

use \Webmors\Catalog\Models\Section;

/**
 * Model
 */
class Product extends Model
{
    use \October\Rain\Database\Traits\Validation;
    
    use \October\Rain\Database\Traits\SoftDelete;

    use \October\Rain\Database\Traits\Sortable;

    use \October\Rain\Database\Traits\Sluggable;
    
    protected $dates = ['deleted_at'];

    protected $slugs = ['slug' => 'title'];

    /**
     * @var string The database table used by the model.
     */
    public $table = 'webmors_catalog_product';

    public $belongsTo = [
        'category' => ['Webmors\Catalog\Models\Category', 'otherKey' => 'id']
    ];
    
    public $hasMany = [
        'sections' => ['Webmors\Catalog\Models\Section', 'key' => 'product_id']
    ];
    
    public $implement = ['RainLab.Translate.Behaviors.TranslatableModel'];
    
    public $translatable = ['title', 'description', 'preview_text', 'specifications', 'benefits', 'manufacturer', 'additional', 'features_repeater', 'accordion_repeater', 'location', 'status', 'preview_image_label']; 
    
    protected $jsonable = ['features_repeater', 'accordion_repeater', 'gallery_repeater'];
    
    /**
     * @var array Validation rules
     */
    public $rules = [
    ];

    /* Scopes */

    // Get the products of a category
    public function scopeProducts($query, $type)
    {
        return $query->where('category_id', $type)->where('is_active', true);
    }
    
    // Get the products of a category
    public function scopeFeatured($query, $type)
    {
        return $query->where('category_id', $type)->where('is_active', true)->where('is_featured', true);
    }
    
    public static function resolveMenuItem($item, $url, $theme)
    {
        $pageName = $item->cmsPage;
        $categoryId = $item->reference;
        $cmsPage = \Cms\Classes\Page::loadCached($theme, $pageName);
        $items   = self
            ::where('category_id',$categoryId)->where('is_active',true)->orderBy('sort_order', 'ASC')
            ->get()
            ->map(function (self $item) use ($cmsPage, $url, $pageName) {
                $pageUrl = $cmsPage->url($pageName, ['catslug' => $item->category->catslug,'slug' => $item->slug]);
                $catslug = $item->category->catslug;
                $itemslug = $item->slug;

                $sections   = Section
                ::where('product_id',$item->id)
                ->join('webmors_catalog_tabsection', 'webmors_catalog_tabsection.id', '=', 'webmors_catalog_section.tabsection_id')
                ->orderBy('webmors_catalog_tabsection.sort_order')
                ->select('webmors_catalog_section.*') //see PS:
                ->get()
                ->map(function (Section $section) use ($cmsPage, $url, $pageName, $catslug, $itemslug) {
                    $pageUrl = $cmsPage->url($pageName, ['catslug' => $catslug,'slug' => $itemslug,'contentslug' => $section->contentslug]);
                      return [
                        'title'    => $section->title,
                        'url'      => $pageUrl,
                        'mtime'    => $section->updated_at,
                        'isActive' => $pageUrl === $url,
                    ];
                })
                ->toArray();
                return [
                    'title'    => $item->title,
                    'items' => $sections,
                    'url'      => $pageUrl,
                    'mtime'    => $item->updated_at,
                    'isActive' => $pageUrl === $url,
                ];
            })
            ->toArray();
    
        return [
            'items' => $items,
        ];
    }
}
